import { useCallback, useEffect, useState } from 'react';
import cx from 'classnames';
import PropTypes from 'prop-types';
import useControlledState from '@s-ui/react-hooks/lib/useControlledState';
import { BASE_CLASS, BUTTON_CLASS, BUTTON_CONTENT_CLASS, BUTTON_TEXT_ALIGN, COLLAPSED_CLASS, CONTAINER_BUTTON_CLASS, CONTENT_ALIGN, CONTENT_CLASS, ICON_CLASS, MIN_HEIGHT } from './settings.js';
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
var MoleculeCollapsible = function MoleculeCollapsible(_ref) {
  var _cx, _cx2, _cx3, _cx4;
  var _ref$onClose = _ref.onClose,
    onClose = _ref$onClose === void 0 ? function () {} : _ref$onClose,
    _ref$onOpen = _ref.onOpen,
    onOpen = _ref$onOpen === void 0 ? function () {} : _ref$onOpen,
    alignButtonText = _ref.alignButtonText,
    alignContainer = _ref.alignContainer,
    children = _ref.children,
    _ref$height = _ref.height,
    height = _ref$height === void 0 ? MIN_HEIGHT : _ref$height,
    icon = _ref.icon,
    _ref$isCollapsible = _ref.isCollapsible,
    isCollapsible = _ref$isCollapsible === void 0 ? true : _ref$isCollapsible,
    _ref$isDefaultExpande = _ref.isDefaultExpanded,
    isDefaultExpanded = _ref$isDefaultExpande === void 0 ? false : _ref$isDefaultExpande,
    isExpanded = _ref.isExpanded,
    showText = _ref.showText,
    hideText = _ref.hideText,
    _ref$withGradient = _ref.withGradient,
    withGradient = _ref$withGradient === void 0 ? true : _ref$withGradient,
    _ref$withOverflow = _ref.withOverflow,
    withOverflow = _ref$withOverflow === void 0 ? false : _ref$withOverflow,
    _ref$withTransition = _ref.withTransition,
    withTransition = _ref$withTransition === void 0 ? true : _ref$withTransition;
  var _useControlledState = useControlledState(isExpanded, isDefaultExpanded),
    expanded = _useControlledState[0],
    setExpanded = _useControlledState[1];
  var _useState = useState(true),
    showButton = _useState[0],
    setShowButton = _useState[1];
  var _useState2 = useState(0),
    childrenHeight = _useState2[0],
    setChildrenHeight = _useState2[1];
  var nodeCallback = useCallback(function (node) {
    setChildrenHeight(node !== null ? node.getBoundingClientRect().height : 0);
  }, [children]);
  var toggleCollapse = function toggleCollapse() {
    if (showButton) {
      setExpanded(!expanded);
      expanded ? onClose() : onOpen();
    }
  };
  useEffect(function () {
    if (!childrenHeight || !expanded) return;
    setShowButton(isCollapsible && childrenHeight >= height);
  }, [childrenHeight, expanded, height, isCollapsible, setShowButton]);
  var wrapperClassName = cx("" + BASE_CLASS, (_cx = {}, _cx[BASE_CLASS + "--withGradient"] = withGradient, _cx[COLLAPSED_CLASS] = !expanded, _cx));
  var iconClassName = cx("" + ICON_CLASS, (_cx2 = {}, _cx2[COLLAPSED_CLASS] = !expanded, _cx2));
  var containerClassName = cx("" + CONTAINER_BUTTON_CLASS, (_cx3 = {}, _cx3[CONTAINER_BUTTON_CLASS + "--withGradient"] = withGradient, _cx3[CONTAINER_BUTTON_CLASS + "--alignButtonText-" + alignButtonText] = alignButtonText, _cx3[COLLAPSED_CLASS] = !expanded, _cx3));
  var contentClassName = cx("" + CONTENT_CLASS, (_cx4 = {}, _cx4[CONTENT_CLASS + "--withTransition"] = withTransition, _cx4[CONTENT_CLASS + "--withOverflow"] = withOverflow, _cx4[CONTENT_CLASS + "--alignContainer-" + alignContainer] = alignContainer, _cx4));
  var containerHeight = !expanded ? height + "px" : childrenHeight + "px";
  return /*#__PURE__*/_jsxs("div", {
    className: wrapperClassName,
    children: [/*#__PURE__*/_jsx("div", {
      className: contentClassName,
      style: {
        maxHeight: !showButton ? 'none' : containerHeight
      },
      children: /*#__PURE__*/_jsx("div", {
        ref: nodeCallback,
        children: children
      })
    }), showButton && /*#__PURE__*/_jsx("div", {
      className: containerClassName,
      children: /*#__PURE__*/_jsx("button", {
        type: "button",
        className: BUTTON_CLASS,
        onClick: toggleCollapse,
        children: /*#__PURE__*/_jsxs("span", {
          className: BUTTON_CONTENT_CLASS,
          tabIndex: "-1",
          children: [!expanded ? showText : hideText, /*#__PURE__*/_jsx("span", {
            className: iconClassName,
            children: icon
          })]
        })
      })
    })]
  });
};
MoleculeCollapsible.displayName = 'MoleculeCollapsible';
export default MoleculeCollapsible;
export { CONTENT_ALIGN, CONTENT_ALIGN as moleculeCollapsibleContentAlign, BUTTON_TEXT_ALIGN as moleculeCollapsibleButtonAlign };